# -*- coding: utf-8 -*-
#
# Copyright (C) 2010 by Intevation GmbH
# Authors:
# Bjoern Ricks <bjoern.ricks@intevation.de>
#
# This program is free software under the GPL (>=v2)
# Read the file COPYING coming with the software for details.

import sys
import os.path

from string import Template
from scm import Svn
from scm import SvnTagDetector
from scm import Git
from scm import GitTagDetector

class PortTemplateException(Exception):
    
    """ Exception class for PortTemplate"""
    
    def __init__(self, msg):
        Exception.__init__(self)
        self.msg = msg
    
    def __str__(self):
        return repr(self.msg)

class PortTemplate:

    """ Class that generates port files from templates and determines the latest
    tag in svn from tagsurl"""

    def __init__(self, portname, templatefile, portfile, tagsurl, tagspattern,
                 tagssubdir, scm):
        if scm == "git":
            tagdetector = GitTagDetector(tagsurl, tagspattern, tagssubdir)
            tagurl = tagsurl
            (tagname, revno, tagsubdir) = tagdetector.newest_tag_revision()
        else:
            tagdetector = SvnTagDetector(tagsurl, tagspattern, tagssubdir)
            (tagurl, revno, tagsubdir) = tagdetector.newest_tag_revision()
            tagname = tagurl
        tagdate = tagdetector.get_tag_date()

        self.subst = dict(tagurl=tagurl, revision=revno, tagsubdir=tagsubdir,
                          tagdate=tagdate, tagname=tagname)
        self.portname = portname
        self.portfile = portfile
        self.filetemplate = self.get_template(templatefile)

    def substitute(self):
        return self.filetemplate.substitute(self.subst)

    def write(self):
        if not self.portfile:
            raise PortTemplateException, "please specify a portfile for "  \
                                          "port %s" % self.portname
        self.substitute()
        self.filetemplate.save(self.portfile)

    def get_template(self, filename):
        if not filename:
            raise PortTemplateException("please specify a templatefile " \
                                         "for port %s" % self.portname)
        if not os.path.exists(filename):
            raise PortTemplateException("template file %s for port %s not " \
                                         "found" % (filename,
                                         self.portname))
        return FileTemplate(filename)


class FileTemplate:
    
    def __init__(self, filename):
        self.filename = filename
        self.load()

    def load(self):
        f = open(self.filename, 'r')
        strtemplate = f.read()
        self.portfile = strtemplate
        self.template = Template(strtemplate)
        f.close()

    def substitute(self, subst):
        self.portfile = self.template.substitute(subst)
        return self.portfile

    def save(self, filename):
        f = open(filename, 'w')
        f.write(self.portfile)
        f.close()

